﻿using GalaSoft.MvvmLight.Command;
using Plane.Copters;
using Plane.Windows.Messages;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTaskSingleCopterInfo
    {
        public static FlightTaskSingleCopterInfo CreateForFlyToTask(ICopter copter, double targetLat, double targetLng, float targetAlt)
        {
            var info = new FlightTaskSingleCopterInfo(copter)
            {
                TargetLat = targetLat,
                TargetLng = targetLng,
                TargetAlt = targetAlt
            };
            return info;
        }

        public static FlightTaskSingleCopterInfo CreateForFlyToTask(ICopter copter, LatLng latLngOffset, float targetAlt, bool isLandWaypoint)
        {
            var info = new FlightTaskSingleCopterInfo(copter)
            {
                LatLngOffset = latLngOffset,
                TargetAlt = targetAlt,
                IsLandWaypoint = isLandWaypoint
            };
            return info;
        }

        public static FlightTaskSingleCopterInfo CreateForFlyToTask(ICopter copter, double x, double y, float targetAlt, bool isLandWaypoint)
        {
            var info = new FlightTaskSingleCopterInfo(copter)
            {
                X = x,
                Y = y,
                TargetAlt = targetAlt,
                IsLandWaypoint = isLandWaypoint
            };
            return info;
        }

    private bool _FlytoShowLED = true;
        public bool FlytoShowLED
        {
            get { return _FlytoShowLED; }
            set { Set(nameof(FlytoShowLED), ref _FlytoShowLED, value); }
        }

        private bool _IsLandWaypoint = false;

        /// <summary>
        /// 降落前的航点 只有高度有效,自动飞往起飞点，写入航点的经纬为90 180 飞控自动认为90 180是飞往起点
        /// </summary>
        public bool IsLandWaypoint
        {
            get { return _IsLandWaypoint; }
            set { Set(nameof(IsLandWaypoint), ref _IsLandWaypoint, value); }
        }

        private bool _IsChangeSpeed = false;

        /// <summary>
        /// 是否在当前航点前修改飞行速度
        /// </summary>
        public bool IsChangeSpeed
        {
            get {return _IsChangeSpeed; }
            set { Set(nameof(IsChangeSpeed), ref _IsChangeSpeed, value); }
        }

       
        private float _LevelSpeed = 5.0f;
        /// <summary>
        /// 水平速度 IsChangeSpeed为True有效
        /// </summary>
        public float LevelSpeed
        {
            get { return _LevelSpeed; }
            set { Set(nameof(LevelSpeed), ref _LevelSpeed, value); }
        }

        
        private float _UpSpeed = 2.5f;
        /// <summary>
        /// 上升速度 IsChangeSpeed为True有效
        /// </summary>
        public float UpSpeed
        {
            get { return _UpSpeed; }
            set { Set(nameof(UpSpeed), ref _UpSpeed, value); }
        }

        private float _DownSpeed = 1.5f;
        /// <summary>
        /// 下降速度 IsChangeSpeed为True有效
        /// </summary>
        public float DownSpeed
        {
            get { return _DownSpeed; }
            set { Set(nameof(DownSpeed), ref _DownSpeed, value); }
        }

        /*
         * //同一个任务每一架飞机的FlytoTime和LoiterTime保持统一,已将FlytoTime和LoiterTime改到FlightTask.cs中
         * 
        private int _FlytoTime = 10;
        public int FlytoTime
        {
            get { return _FlytoTime; }
            set { Set(nameof(FlytoTime), ref _FlytoTime, value); }
        }
        private int _LoiterTime = 0;
        public int LoiterTime
        {
            get { return _LoiterTime; }
            set { Set(nameof(LoiterTime), ref _LoiterTime, value); }
        }
        */



    }
}
