﻿using GalaSoft.MvvmLight.Command;
using Plane.Copters;
using Plane.Windows.Messages;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace Plane.FormationCreator.Formation
{
    public partial class FlightTaskSingleCopterInfo
    {
        public static FlightTaskSingleCopterInfo CreateForFlyToTask(ICopter copter)
        {
            var info = new FlightTaskSingleCopterInfo(copter)
            {
               
            };
            return info;
        }

        public void  AddLEDInfo()
        {       
            LEDInfo LedInfo = new LEDInfo();
            LEDInfos.Add(LedInfo);
            LedInfo.RemoveEvent += new LEDInfo.RemoveEventHandler(LEDRemove);          
        }

        public void AddLEDInfo(LEDInfo LedInfo)
        {
            LEDInfos.Add(LedInfo);
            LedInfo.RemoveEvent += new LEDInfo.RemoveEventHandler(LEDRemove);
        }

        public void LEDRemove(object sender)
        {
            LEDInfo LedInfo = sender as LEDInfo;
            LEDInfos.Remove(LedInfo);
        }

     
        public ObservableCollection<LEDInfo> LEDInfos { get; } = new ObservableCollection<LEDInfo>();


        //设置飞机航点中灯光
        private ICommand _AddLEDCommand;
        public ICommand AddLEDCommand
        {
            get
            {
                return _AddLEDCommand ?? (_AddLEDCommand = new RelayCommand<double>(async =>
                {
                    AddLEDInfo();
                }));
            }
        }

        private ICommand _SetAllCopterLEDCommand;
        public ICommand SetAllCopterLEDCommand
        {
            get
            {
                return _SetAllCopterLEDCommand ?? (_SetAllCopterLEDCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (!info.Equals(this))
                        {
                            info.LEDInfos.Clear();
                            foreach (LEDInfo led in LEDInfos)
                            {
                                LEDInfo addLED = new LEDInfo();
                                addLED.CopyForm(led);
                                info.AddLEDInfo(addLED);
                            }
                        }
                    }
                }));
            }
        }



        private ICommand _SetSelectedCopterLEDCommand;
        public ICommand SetSelectedCopterLEDCommand
        {
            get
            {
                return _SetSelectedCopterLEDCommand ?? (_SetSelectedCopterLEDCommand = new RelayCommand<double>(async =>
                {
                    foreach (FlightTaskSingleCopterInfo info in _flightTaskManager.SelectedTask.SingleCopterInfos)
                    {
                        if (_copterManager.AcceptingControlCopters.Contains(info.Copter))
                        {
                            if (!info.Equals(this))
                            {
                                info.LEDInfos.Clear();
                                foreach (LEDInfo led in LEDInfos)
                                {
                                    LEDInfo addLED = new LEDInfo();
                                    addLED.CopyForm(led);
                                    info.AddLEDInfo(addLED);
                                }
                            }
                        }
                    }
                }));
            }
        }
    }

    public class LEDInfo
    {
        private double delay = 1;    //灯光延时  单位：100ms  （从当前任务开为0，延时几秒后设置LED）
        private int ledmode = 0;     //灯光模式 0常亮 1闪烁 2随机闪烁(RGB无意义)
        private int ledrate = 0;     //闪烁延时  单位：100ms
        private int ledtimes = 0;    //次数 (预留、暂取消其意义)
        private string ledRGB = "FFFFFF";      //灯光颜色

        public LEDInfo()
        {

        }

        public void CopyForm(LEDInfo led)
        {
            delay = led.delay;
            ledmode = led.ledmode;
            ledrate = led.ledrate;
            ledRGB = led.ledRGB;
        }
        public double Delay
        {
            get { return delay; }
            set { delay = value; }
        }

        public int LEDMode
        {
            get { return ledmode; }
            set { ledmode = value; }
        }
        public int LEDRate
        {
            get { return ledrate; }
            set { ledrate = value; }
        }
        public int LEDTimes
        {
            get { return ledtimes; }
            set { ledtimes = value; }
        }

        public string LEDRGB
        {
            get { return ledRGB; }
            set { ledRGB = value; }
        }

        public delegate void RemoveEventHandler(object sender);
        public event RemoveEventHandler RemoveEvent;
        private ICommand _RemoveLEDCommand;
        [Newtonsoft.Json.JsonIgnore]
        public ICommand RemoveLEDCommand
        {
            get
            {
                return _RemoveLEDCommand ?? (_RemoveLEDCommand = new RelayCommand<double>(async =>
                {
                    RemoveEvent(this);
                }));
            }
        }
    }
}
